# E-mail

Un mesaj e-mail a fost întotdeauna transmis în format plain-text (text
clar). Chiar si prin adăugarea atașamentelor, mesajele de e-mail sunt
trimise tot ca mesaje plain-text, prin folosirea unor mecanisme de
codificare (*uuencode/uudecode*, *MIME/BASE64*).

Un mesaj este format dintr-o secțiune de antete (headers), urmată de o
secțiune cu conținutul mesajului. Structura antetelor este descrisă în
[RFC 822](https://tools.ietf.org/html/rfc822), [RFC
1521](https://tools.ietf.org/html/rfc1521) și [RFC
1806](https://tools.ietf.org/html/rfc1806), ele având în general
următoarea structură:

-   unul sau mai multe antete *Received*, care indică ce cale a fost
    urmată de mesaj de la sursa până la destinație
-   *Mime-Version*: versiunea MIME (Multipurpose Internet Mail
    Extensions) folosită, 1.0 in general
-   *Content-Type*: *text/plain* pentru mesaje text, *multipart/mixed*
    pentru mesaje cu atașamente
-   *Subject*: subiectul mesajului
-   *Date*: data și ora când a fost trimis mesajul
-   *Message ID*: un ID pentru mesaj, folosit pentru identificarea în
    mod unic a unui mesaj
-   *From*: numele și adresa de mail a expeditorului
-   *To*: numele și adresa de mail a destinatarului
-   *Cc*: carbon copy (alți destinatari)
-   alte antete introduse de clientul de e-mail folosit pentru a trimite
    mesajul.

Conținutul mesajului este textul propriu-zis, pentru mesajele în text
clar fără atașamente. Se poate observa mai jos un exemplu de mesaj:

    MIME-Version: 1.0
    From: profesor@upb.ro
    To: student@upb.ro
    Subject: Tema
    Content-Type: text/plain

    Draga student,

    Fa-ti tema!

    Cu bine,
    Profesorul.

Mesajele cu atașamente pot folosi una din următoarele tehnici pentru
codificarea acestora:

-   *uuencode* - la începuturile e-mail-ului, fișierele care se doreau
    trimise trebuiau convertite în format text și invers prin folosirea
    utilitarelor numite *uuencode/uudecode*; și în ziua de azi, unii
    clienți de mail adaugă atașamentele la sfârșitul mesajelor,
    codificându-le cu algoritmul folosit de *uuencode*
-   *MIME/Base64* - această tehnologie este cea recomandată pentru
    trimiterea de mesaje cu atașamente.

Un mesaj cu atașamente codificate
[MIME](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types)
arată în felul următor:

    MIME-Version: 1.0
    From: Student Studentescu <student@upb.ro>
    To: Profesor PC <profesor@upb.ro>
    Subject: Re: Tema
    Content-Type: multipart/mixed; boundary=abc

    --abc
    Content-Type: text/plain

    Atasez tema.

    Cu bine,
    Studentul

    --abc
    Content-Type: text/plain
    Content-Disposition: attachment; filename="tema.c"

    #include <stdio.h>
    int main()
    {
        printf("Aceasta este tema mea\n");
        return 0;
    }
    --abc

Se observă faptul că părțile care compun mesajul sunt separate între ele
printr-un șir de caractere separator (boundary string), specificat ca un
parametru pentru antetul *Content-Type*. Fiecare parte poate avea la
rândul ei propriile antete, care conțin în general tipul și numele
fișierului din secțiunea respectivă. În cazul în care se trimit
atașamente binare, acestea sunt codificate folosind schema numită
Base64, descrisă în [RFC 1521](https://tools.ietf.org/html/rfc1521).
